# File which defines all remote URLs

server = 'https://ftp.gajim.org'
directory = 'plugins_1.3_zip'

PLUGINS_DIR_URL = '%s/%s' % (server, directory)
MANIFEST_URL = '%s/manifests.zip' % PLUGINS_DIR_URL
MANIFEST_IMAGE_URL = '%s/manifests_images.zip' % PLUGINS_DIR_URL
